@@ -1,7 +1,7 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
 //
-// Copyright 2009-2017 by Wilson Snyder. This program is free software; you can
+// Copyright 2009-2019 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License.
 // Version 2.0.
@@ -33,8 +33,8 @@
 #include "verilated_vpi.h"
 
 #include <list>
-#include <set>
 #include <map>
+#include <set>
 
 //======================================================================
 // Internal constants
@@ -79,11 +79,10 @@ public:
             vluint8_t* newp = t_freeHead;
             t_freeHead = *((vluint8_t**)newp);
             return newp+8;
-        } else {
-            // +8: 8 bytes for next
-            vluint8_t* newp = reinterpret_cast<vluint8_t*>(::operator new(chunk+8));
-            return newp+8;
         }
+        // +8: 8 bytes for next
+        vluint8_t* newp = reinterpret_cast<vluint8_t*>(::operator new(chunk+8));
+        return newp+8;
     }
     inline static void operator delete(void* obj, size_t size) VL_MT_SAFE {
         vluint8_t* oldp = ((vluint8_t*)obj)-8;
@@ -91,7 +90,8 @@ public:
         t_freeHead = oldp;
     }
     // MEMBERS
-    static inline VerilatedVpio* castp(vpiHandle h) { return dynamic_cast<VerilatedVpio*>((VerilatedVpio*)h); }
+    static inline VerilatedVpio* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpio*>((VerilatedVpio*)h); }
     inline vpiHandle castVpiHandle() { return reinterpret_cast<vpiHandle>(this); }
     // ACCESSORS
     virtual const char* name() const { return "<null>"; }
@@ -117,7 +117,8 @@ public:
         m_cbData.value = &m_value;
     }
     virtual ~VerilatedVpioCb() {}
-    static inline VerilatedVpioCb* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioCb*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioCb* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioCb*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiCallback; }
     vluint32_t reason() const { return m_cbData.reason; }
     VerilatedPliCb cb_rtnp() const { return m_cbData.cb_rtn; }
@@ -130,7 +131,8 @@ class VerilatedVpioConst : public VerilatedVpio {
 public:
     explicit VerilatedVpioConst(vlsint32_t num) : m_num(num) {}
     virtual ~VerilatedVpioConst() {}
-    static inline VerilatedVpioConst* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioConst*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioConst* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioConst*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiUndefined; }
     vlsint32_t num() const { return m_num; }
 };
@@ -141,7 +143,8 @@ class VerilatedVpioRange : public VerilatedVpio {
 public:
     explicit VerilatedVpioRange(const VerilatedRange* range) : m_range(range), m_iteration(0) {}
     virtual ~VerilatedVpioRange() {}
-    static inline VerilatedVpioRange* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioRange*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioRange* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioRange*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiRange; }
     virtual vluint32_t size() const { return m_range->elements(); }
     virtual const VerilatedRange* rangep() const { return m_range; }
@@ -152,9 +155,8 @@ public:
             VerilatedVpioRange* nextp = new VerilatedVpioRange(*this);
             nextp->iterationInc();
             return ((nextp)->castVpiHandle());
-        } else {
-            return 0;  // End of list - only one deep
         }
+        return 0;  // End of list - only one deep
     }
 };
 
@@ -164,7 +166,8 @@ public:
     explicit VerilatedVpioScope(const VerilatedScope* scopep)
         : m_scopep(scopep) {}
     virtual ~VerilatedVpioScope() {}
-    static inline VerilatedVpioScope* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioScope*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioScope* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioScope*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiScope; }
     const VerilatedScope* scopep() const { return m_scopep; }
     virtual const char* name() const { return m_scopep->name(); }
@@ -185,20 +188,21 @@ protected:
     vlsint32_t                  m_index;
     const VerilatedRange&       get_range() const {
         // Determine number of dimensions and return outermost
-        return (m_varp->dims()>1) ? m_varp->array() : m_varp->range();
+        return (m_varp->dims()>1) ? m_varp->unpacked() : m_varp->packed();
     }
 public:
     VerilatedVpioVar(const VerilatedVar* varp, const VerilatedScope* scopep)
         : m_varp(varp), m_scopep(scopep), m_index(0) {
         m_prevDatap = NULL;
-        m_mask.u32 = VL_MASK_I(varp->range().elements());
+        m_mask.u32 = VL_MASK_I(varp->packed().elements());
         m_entSize = varp->entSize();
         m_varDatap = varp->datap();
     }
     virtual ~VerilatedVpioVar() {
         if (m_prevDatap) { delete [] m_prevDatap; m_prevDatap = NULL; }
     }
-    static inline VerilatedVpioVar* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioVar*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioVar* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioVar*>((VerilatedVpio*)h); }
     const VerilatedVar* varp() const { return m_varp; }
     const VerilatedScope* scopep() const { return m_scopep; }
     vluint32_t mask() const { return m_mask.u32; }
@@ -236,10 +240,11 @@ public:
         m_varDatap = ((vluint8_t*)varp->datap()) + entSize()*offset;
     }
     virtual ~VerilatedVpioMemoryWord() {}
-    static inline VerilatedVpioMemoryWord* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioMemoryWord*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioMemoryWord* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioMemoryWord*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiMemoryWord; }
-    virtual  vluint32_t size() const { return varp()->range().elements(); }
-    virtual const VerilatedRange* rangep() const { return &(varp()->range()); }
+    virtual  vluint32_t size() const { return varp()->packed().elements(); }
+    virtual const VerilatedRange* rangep() const { return &(varp()->packed()); }
     virtual const char* fullname() const {
         static VL_THREAD_LOCAL std::string out;
         char num[20]; sprintf(num,"%d",m_index);
@@ -256,7 +261,8 @@ public:
     explicit VerilatedVpioVarIter(const VerilatedScope* scopep)
         : m_scopep(scopep), m_started(false) {  }
     virtual ~VerilatedVpioVarIter() {}
-    static inline VerilatedVpioVarIter* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioVarIter*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioVarIter* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioVarIter*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiIterator; }
     virtual vpiHandle dovpi_scan() {
         if (VL_LIKELY(m_scopep->varsp())) {
@@ -267,9 +273,8 @@ public:
             if (m_it == varsp->end()) return 0;
             return ((new VerilatedVpioVar(&(m_it->second), m_scopep))
                     ->castVpiHandle());
-        } else {
-            return 0;  // End of list - only one deep
         }
+        return 0;  // End of list - only one deep
     }
 };
 
@@ -281,12 +286,16 @@ class VerilatedVpioMemoryWordIter : public VerilatedVpio {
     bool                        m_done;
 public:
     VerilatedVpioMemoryWordIter(const vpiHandle handle, const VerilatedVar* varp)
-        : m_handle(handle), m_varp(varp), m_iteration(varp->array().right()), m_direction(VL_LIKELY(varp->array().left()>varp->array().right())?1:-1), m_done(false) {  }
+        : m_handle(handle), m_varp(varp), m_iteration(varp->unpacked().right()),
+          m_direction(VL_LIKELY(varp->unpacked().left() > varp->unpacked().right())
+                      ? 1 : -1),
+          m_done(false) { }
     virtual ~VerilatedVpioMemoryWordIter() {}
-    static inline VerilatedVpioMemoryWordIter* castp(vpiHandle h) { return dynamic_cast<VerilatedVpioMemoryWordIter*>((VerilatedVpio*)h); }
+    static inline VerilatedVpioMemoryWordIter* castp(vpiHandle h) {
+        return dynamic_cast<VerilatedVpioMemoryWordIter*>((VerilatedVpio*)h); }
     virtual vluint32_t type() const { return vpiIterator; }
     void iterationInc() {
-        if (!(m_done = (m_iteration == m_varp->array().left()))) m_iteration+=m_direction;
+        if (!(m_done = (m_iteration == m_varp->unpacked().left()))) m_iteration+=m_direction;
     }
     virtual vpiHandle dovpi_scan() {
         vpiHandle result;
@@ -337,7 +346,9 @@ public:
                 varop->createPrevDatap();
             }
         }
-        if (VL_UNLIKELY(vop->reason() >= CB_ENUM_MAX_VALUE)) VL_FATAL_MT(__FILE__,__LINE__,"", "vpi bb reason too large");
+        if (VL_UNLIKELY(vop->reason() >= CB_ENUM_MAX_VALUE)) {
+            VL_FATAL_MT(__FILE__,__LINE__,"", "vpi bb reason too large");
+        }
         s_s.m_cbObjLists[vop->reason()].push_back(vop);
     }
     static void cbTimedAdd(VerilatedVpioCb* vop) {
@@ -374,9 +385,8 @@ public:
         VpioTimedCbs::const_iterator it=s_s.m_timedCbs.begin();
         if (VL_LIKELY(it!=s_s.m_timedCbs.end())) {
             return it->first;
-        } else {
-            return ~VL_ULL(0);  // maxquad
         }
+        return ~VL_ULL(0);  // maxquad
     }
     static void callCbs(vluint32_t reason) {
         VpioCbList& cbObjList = s_s.m_cbObjLists[reason];
@@ -459,11 +469,12 @@ public:
         m_errorInfo.level = level;
         return this;
     }
-    void setMessage(std::string file, PLI_INT32 line, std::string message, ...) {
+    void setMessage(std::string file, PLI_INT32 line, const char* message, ...) {
+        // message cannot be a const string& as va_start cannot use a reference
         static VL_THREAD_LOCAL std::string filehold;
         va_list args;
         va_start(args, message);
-        VL_VSNPRINTF(m_buff, sizeof(m_buff), message.c_str(), args);
+        VL_VSNPRINTF(m_buff, sizeof(m_buff), message, args);
         va_end(args);
         m_errorInfo.state = vpiPLI;
         filehold = file;
@@ -910,7 +921,7 @@ void VerilatedVpiError::selfTest() VL_MT_UNSAFE_ONE {
 
 vpiHandle vpi_register_cb(p_cb_data cb_data_p) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     // cppcheck-suppress nullPointer
     if (VL_UNLIKELY(!cb_data_p)) {
         _VL_VPI_WARNING(__FILE__, __LINE__, "%s : callback data pointer is null", VL_FUNC);
@@ -921,7 +932,8 @@ vpiHandle vpi_register_cb(p_cb_data cb_data_p) {
 	QData time = 0;
 	if (cb_data_p->time) time = _VL_SET_QII(cb_data_p->time->high, cb_data_p->time->low);
 	VerilatedVpioCb* vop = new VerilatedVpioCb(cb_data_p, VL_TIME_Q()+time);
-	VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_register_cb %d %p delay=%" VL_PRI64 "u\n",cb_data_p->reason,vop,time););
+        VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_register_cb %d %p delay=%" VL_PRI64 "u\n",
+                                    cb_data_p->reason, vop, time););
 	VerilatedVpiImp::cbTimedAdd(vop);
 	return vop->castVpiHandle();
     }
@@ -951,7 +963,7 @@ PLI_INT32 vpi_remove_cb(vpiHandle object) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_remove_cb %p\n",object););
     VerilatedVpiImp::assertOneCheck();
     VerilatedVpioCb* vop = VerilatedVpioCb::castp(object);
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!vop)) return 0;
     if (vop->cb_datap()->reason == cbAfterDelay) {
 	VerilatedVpiImp::cbTimedRemove(vop);
@@ -975,7 +987,7 @@ void vpi_get_systf_info(vpiHandle object, p_vpi_systf_data systf_data_p) {
 
 vpiHandle vpi_handle_by_name(PLI_BYTE8* namep, vpiHandle scope) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!namep)) return NULL;
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_handle_by_name %s %p\n",namep,scope););
     VerilatedVpioScope* voScopep = VerilatedVpioScope::castp(scope);
@@ -1012,24 +1024,24 @@ vpiHandle vpi_handle_by_index(vpiHandle object, PLI_INT32 indx) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_handle_by_index %p %d\n",object, indx););
     VerilatedVpiImp::assertOneCheck();
     VerilatedVpioVar* varop = VerilatedVpioVar::castp(object);
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_LIKELY(varop)) {
 	if (varop->varp()->dims()<2) return 0;
-	if (VL_LIKELY(varop->varp()->array().left() >= varop->varp()->array().right())) {
-	    if (VL_UNLIKELY(indx > varop->varp()->array().left() || indx < varop->varp()->array().right())) return 0;
+        if (VL_LIKELY(varop->varp()->unpacked().left() >= varop->varp()->unpacked().right())) {
+            if (VL_UNLIKELY(indx > varop->varp()->unpacked().left()
+                            || indx < varop->varp()->unpacked().right())) return 0;
 	    return (new VerilatedVpioMemoryWord(varop->varp(), varop->scopep(), indx,
-					      indx - varop->varp()->array().right()))
-		->castVpiHandle();
-	} else {
-	    if (VL_UNLIKELY(indx < varop->varp()->array().left() || indx > varop->varp()->array().right())) return 0;
-	    return (new VerilatedVpioMemoryWord(varop->varp(), varop->scopep(), indx,
-					      indx - varop->varp()->array().left()))
+                                              indx - varop->varp()->unpacked().right()))
 		->castVpiHandle();
 	}
-    } else {
-	_VL_VPI_INTERNAL(__FILE__, __LINE__, "%s : can't resolve handle", VL_FUNC);
-        return 0;
+        if (VL_UNLIKELY(indx < varop->varp()->unpacked().left()
+                        || indx > varop->varp()->unpacked().right())) return 0;
+        return (new VerilatedVpioMemoryWord(varop->varp(), varop->scopep(), indx,
+                                            indx - varop->varp()->unpacked().left()))
+            ->castVpiHandle();
     }
+    _VL_VPI_INTERNAL(__FILE__, __LINE__, "%s : can't resolve handle", VL_FUNC);
+    return 0;
 }
 
 // for traversing relationships
@@ -1037,7 +1049,7 @@ vpiHandle vpi_handle_by_index(vpiHandle object, PLI_INT32 indx) {
 vpiHandle vpi_handle(PLI_INT32 type, vpiHandle object) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_handle %d %p\n",type,object););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     switch (type) {
     case vpiLeftRange: {
 	VerilatedVpioVar* vop = VerilatedVpioVar::castp(object);
@@ -1080,7 +1092,7 @@ vpiHandle vpi_handle_multi(PLI_INT32 type, vpiHandle refHandle1, vpiHandle refHa
 vpiHandle vpi_iterate(PLI_INT32 type, vpiHandle object) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_iterate %d %p\n",type,object););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     switch (type) {
     case vpiMemoryWord: {
 	VerilatedVpioVar* vop = VerilatedVpioVar::castp(object);
@@ -1118,7 +1130,7 @@ vpiHandle vpi_iterate(PLI_INT32 type, vpiHandle object) {
 vpiHandle vpi_scan(vpiHandle object) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_scan %p\n",object););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     VerilatedVpio* vop = VerilatedVpio::castp(object);
     if (VL_UNLIKELY(!vop)) return NULL;
     return vop->dovpi_scan();
@@ -1130,7 +1142,7 @@ PLI_INT32 vpi_get(PLI_INT32 property, vpiHandle object) {
     // Leave this in the header file - in many cases the compiler can constant propagate "object"
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_get %d %p\n",property,object););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     switch (property) {
     case vpiTimePrecision: {
 	return VL_TIME_PRECISION;
@@ -1173,7 +1185,7 @@ PLI_BYTE8 *vpi_get_str(PLI_INT32 property, vpiHandle object) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_get_str %d %p\n",property,object););
     VerilatedVpiImp::assertOneCheck();
     VerilatedVpio* vop = VerilatedVpio::castp(object);
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!vop)) return NULL;
     switch (property) {
     case vpiName: {
@@ -1206,12 +1218,13 @@ void vpi_put_delays(vpiHandle object, p_vpi_delay delay_p) {
 // value processing
 
 void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
-    static VL_THREAD_LOCAL char outStr[1+VL_MULS_MAX_WORDS*32]; // Maximum required size is for binary string, one byte per bit plus null termination
+    // Maximum required size is for binary string, one byte per bit plus null termination
+    static VL_THREAD_LOCAL char outStr[1+VL_MULS_MAX_WORDS*32];
     // cppcheck-suppress variableScope
     static VL_THREAD_LOCAL int outStrSz = sizeof(outStr)-1;
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_get_value %p\n",object););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!value_p)) return;
     if (VerilatedVpioVar* vop = VerilatedVpioVar::castp(object)) {
 	// We used to presume vpiValue.format = vpiIntVal or if single bit vpiScalarVal
@@ -1235,7 +1248,7 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 		out[0].bval = 0;
 		return;
 	    case VLVT_WDATA: {
-		int words = VL_WORDS_I(vop->varp()->range().elements());
+                int words = VL_WORDS_I(vop->varp()->packed().elements());
 		if (VL_UNLIKELY(words >= VL_MULS_MAX_WORDS)) {
 		    VL_FATAL_MT(__FILE__,__LINE__,"", "vpi_get_value with more than VL_MULS_MAX_WORDS; increase and recompile");
 		}
@@ -1268,25 +1281,28 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int bits = vop->varp()->range().elements();
+                int bits = vop->varp()->packed().elements();
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
 		int i;
 		if (bits > outStrSz) {
 		  // limit maximum size of output to size of buffer to prevent overrun.
 		  bits = outStrSz;
-		  _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
-				  VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, bits);
+                  _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s"
+                                  " as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
+                                  VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format),
+                                  vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, bits);
 		}
 		for (i=0; i<bits; ++i) {
 		    char val = (datap[i>>3]>>(i&7))&1;
 		    outStr[bits-i-1] = val?'1':'0';
-		}
-		outStr[i]=0; // NULL terminate
-		return;
-	    }
+                }
+                outStr[i] = '\0';
+                return;
+            }
 	    default:
 		_VL_VPI_ERROR(__FILE__, __LINE__, "%s: Unsupported format (%s) for %s",
-			      VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
+                              VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format),
+                              vop->fullname());
 		return;
 	    }
 	} else if (value_p->format == vpiOctStrVal) {
@@ -1297,14 +1313,16 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int chars = (vop->varp()->range().elements()+2)/3;
-		int bytes = VL_BYTES_I(vop->varp()->range().elements());
+                int chars = (vop->varp()->packed().elements()+2)/3;
+                int bytes = VL_BYTES_I(vop->varp()->packed().elements());
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
 		int i;
 		if (chars > outStrSz) {
 		    // limit maximum size of output to size of buffer to prevent overrun.
-		    _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
-				    VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, chars);
+                    _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s"
+                                    " as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
+                                    VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format),
+                                    vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, chars);
 		    chars = outStrSz;
 	        }
 		for (i=0; i<chars; ++i) {
@@ -1320,17 +1338,17 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
                     if (i==(chars-1)) {
 			// most signifcant char, mask off non existant bits when vector
                         // size is not a multiple of 3
-			unsigned int rem = vop->varp()->range().elements() % 3;
+                        unsigned int rem = vop->varp()->packed().elements() % 3;
                         if (rem) {
 			    // generate bit mask & zero non existant bits
                             val &= (1<<rem)-1;
 			}
 		    }
 		    outStr[chars-i-1] = '0' + (val&7);
-		}
-		outStr[i]=0; // NULL terminate
-		return;
-	    }
+                }
+                outStr[i] = '\0';
+                return;
+            }
 	    default:
                 strcpy(outStr, "0");
 		_VL_VPI_ERROR(__FILE__, __LINE__, "%s: Unsupported format (%s) for %s",
@@ -1363,13 +1381,15 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int chars = (vop->varp()->range().elements()+3)>>2;
+                int chars = (vop->varp()->packed().elements()+3)>>2;
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
 		int i;
 		if (chars > outStrSz) {
 		  // limit maximum size of output to size of buffer to prevent overrun.
-		  _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
-				  VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, chars);
+                  _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s"
+                                  " as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
+                                  VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format),
+                                  vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, chars);
 		  chars = outStrSz;
 		}
 		for (i=0; i<chars; ++i) {
@@ -1377,17 +1397,17 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
                     if (i==(chars-1)) {
 			// most signifcant char, mask off non existant bits when vector
                         // size is not a multiple of 4
-			unsigned int rem = vop->varp()->range().elements() & 3;
+                        unsigned int rem = vop->varp()->packed().elements() & 3;
                         if (rem) {
 			    // generate bit mask & zero non existant bits
                             val &= (1<<rem)-1;
 			}
 		    }
 		    outStr[chars-i-1] = "0123456789abcdef"[static_cast<int>(val)];
-		}
-		outStr[i]=0; // NULL terminate
-		return;
-	    }
+                }
+                outStr[i] = '\0';
+                return;
+            }
 	    default:
 		_VL_VPI_ERROR(__FILE__, __LINE__, "%s: Unsupported format (%s) for %s",
 			      VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
@@ -1401,12 +1421,13 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int bytes = VL_BYTES_I(vop->varp()->range().elements());
+                int bytes = VL_BYTES_I(vop->varp()->packed().elements());
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
 		int i;
 		if (bytes > outStrSz) {
 		  // limit maximum size of output to size of buffer to prevent overrun.
-		  _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
+                  _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Truncating string value of %s for %s"
+                                  " as buffer size (%d, VL_MULS_MAX_WORDS=%d) is less than required (%d)",
 				  VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format),
 				  vop->fullname(), outStrSz, VL_MULS_MAX_WORDS, bytes);
 		  bytes = outStrSz;
@@ -1415,10 +1436,10 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 		    char val = datap[bytes-i-1];
                      // other simulators replace [leading?] zero chars with spaces, replicate here.
 		    outStr[i] = val?val:' ';
-		}
-		outStr[i]=0; // NULL terminate
-		return;
-	    }
+                }
+                outStr[i] = '\0';
+                return;
+            }
 	    default:
 		_VL_VPI_ERROR(__FILE__, __LINE__, "%s: Unsupported format (%s) for %s",
 			      VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
@@ -1435,9 +1456,9 @@ void vpi_get_value(vpiHandle object, p_vpi_value value_p) {
 	    case VLVT_UINT32:
 		value_p->value.integer = *(reinterpret_cast<IData*>(vop->varDatap()));
 		return;
-	    case VLVT_WDATA: // FALLTHRU
-	    case VLVT_UINT64: // FALLTHRU
-	    default:
+            case VLVT_WDATA:  // FALLTHRU
+            case VLVT_UINT64:  // FALLTHRU
+            default:
 		value_p->value.integer = 0;
                 _VL_VPI_ERROR(__FILE__, __LINE__, "%s: Unsupported format (%s) for %s",
 			      VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
@@ -1467,7 +1488,7 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 			p_vpi_time time_p, PLI_INT32 flags) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_put_value %p %p\n",object, value_p););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!value_p)) {
       _VL_VPI_WARNING(__FILE__, __LINE__, "Ignoring vpi_put_value with NULL value pointer");
       return 0;
@@ -1478,7 +1499,9 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 			VL_DBG_MSGF("- vpi:   varp=%p  putatp=%p\n",
 				     vop->varp()->datap(), vop->varDatap()););
 	if (VL_UNLIKELY(!vop->varp()->isPublicRW())) {
-            _VL_VPI_WARNING(__FILE__, __LINE__, "Ignoring vpi_put_value to signal marked read-only, use public_flat_rw instead: ", vop->fullname());
+            _VL_VPI_WARNING(__FILE__, __LINE__, "Ignoring vpi_put_value to signal marked read-only,"
+                            " use public_flat_rw instead: ",
+                            vop->fullname());
 	    return 0;
 	}
 	if (value_p->format == vpiVectorVal) {
@@ -1494,7 +1517,7 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 		*(reinterpret_cast<IData*>(vop->varDatap())) = value_p->value.vector[0].aval & vop->mask();
 		return object;
 	    case VLVT_WDATA: {
-		int words = VL_WORDS_I(vop->varp()->range().elements());
+                int words = VL_WORDS_I(vop->varp()->packed().elements());
 		WDataOutP datap = (reinterpret_cast<IData*>(vop->varDatap()));
 		for (int i=0; i<words; ++i) {
 		    datap[i] = value_p->value.vector[i].aval;
@@ -1523,7 +1546,7 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int bits = vop->varp()->range().elements();
+                int bits = vop->varp()->packed().elements();
 		int len	 = strlen(value_p->value.str);
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
 		for (int i=0; i<bits; ++i) {
@@ -1550,13 +1573,13 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int chars = (vop->varp()->range().elements()+2)/3;
-		int bytes = VL_BYTES_I(vop->varp()->range().elements());
+                int chars = (vop->varp()->packed().elements()+2)/3;
+                int bytes = VL_BYTES_I(vop->varp()->packed().elements());
 		int len	 = strlen(value_p->value.str);
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
                 div_t idx;
-                datap[0] = 0; // reset zero'th byte
-		for (int i=0; i<chars; ++i) {
+                datap[0] = 0;  // reset zero'th byte
+                for (int i=0; i<chars; ++i) {
                     union {
 			char  byte[2];
                         short half;
@@ -1582,11 +1605,12 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
                     // signifcant byte of the aligned value is the most significant
                     // byte of the destination.
                     val.half <<= idx.rem;
-                    datap[idx.quot] |= val.byte[0]; // or in value
+                    datap[idx.quot] |= val.byte[0];  // or in value
                     if ((idx.quot+1) < bytes) {
-		        datap[idx.quot+1] = val.byte[1]; // this also resets all bits to 0 prior to or'ing above
-		    }
-		}
+                        datap[idx.quot+1] = val.byte[1];  // this also resets
+			// all bits to 0 prior to or'ing above
+                    }
+                }
                 // mask off non existant bits in the most significant byte
                 if (idx.quot == (bytes-1)) {
                     datap[idx.quot] &= vop->mask_byte(idx.quot);
@@ -1615,7 +1639,9 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 	    }
             if (success > 1) {
 		_VL_VPI_WARNING(__FILE__, __LINE__, "%s: Trailing garbage '%s' in '%s' as value %s for %s",
-				VL_FUNC, remainder, value_p->value.str, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
+                                VL_FUNC, remainder, value_p->value.str,
+                                VerilatedVpiError::strFromVpiVal(value_p->format),
+                                vop->fullname());
 	    }
 	    switch (vop->varp()->vltype()) {
 	    case VLVT_UINT8 : *(reinterpret_cast<CData*>(vop->varDatap())) = val & vop->mask(); break;
@@ -1637,7 +1663,7 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int chars = (vop->varp()->range().elements()+3)>>2;
+                int chars = (vop->varp()->packed().elements()+3)>>2;
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
                 char* val = value_p->value.str;
                 // skip hex ident if one is detected at the start of the string
@@ -1655,7 +1681,9 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
                         else if (digit >= 'A' && digit <= 'F') hex = digit - 'A' + 10;
                         else {
 			     _VL_VPI_WARNING(__FILE__, __LINE__, "%s: Non hex character '%c' in '%s' as value %s for %s",
-					     VL_FUNC, digit, value_p->value.str, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
+                                             VL_FUNC, digit, value_p->value.str,
+                                             VerilatedVpiError::strFromVpiVal(value_p->format),
+                                             vop->fullname());
 			    hex = 0;
 			}
 		    } else {
@@ -1665,9 +1693,10 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 		    if (i&1) {
 		        datap[i>>1] |= hex<<4;
 		    } else {
-		        datap[i>>1]  = hex; // this also resets all bits to 0 prior to or'ing above of the msb
-		    }
-		}
+                        datap[i>>1]  = hex;  // this also resets all
+			// bits to 0 prior to or'ing above of the msb
+                    }
+                }
                 // apply bit mask to most significant byte
                 datap[(chars-1)>>1] &= vop->mask_byte((chars-1)>>1);
 		return object;
@@ -1684,11 +1713,12 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 	    case VLVT_UINT32:
 	    case VLVT_UINT64:
 	    case VLVT_WDATA: {
-		int bytes = VL_BYTES_I(vop->varp()->range().elements());
+                int bytes = VL_BYTES_I(vop->varp()->packed().elements());
 		int len	  = strlen(value_p->value.str);
 		CData* datap = (reinterpret_cast<CData*>(vop->varDatap()));
 		for (int i=0; i<bytes; ++i) {
-		    datap[i] = (i < len)?value_p->value.str[len-i-1]:0; // prepend with 0 values before placing string the least signifcant bytes
+                    // prepend with 0 values before placing string the least signifcant bytes
+                    datap[i] = (i < len)?value_p->value.str[len-i-1]:0;
 		}
 		return object;
 	    }
@@ -1708,9 +1738,9 @@ vpiHandle vpi_put_value(vpiHandle object, p_vpi_value value_p,
 	    case VLVT_UINT32:
 		*(reinterpret_cast<IData*>(vop->varDatap())) = vop->mask() & value_p->value.integer;
 		return object;
-	    case VLVT_WDATA: // FALLTHRU
-	    case VLVT_UINT64: // FALLTHRU
-	    default:
+            case VLVT_WDATA:  // FALLTHRU
+            case VLVT_UINT64:  // FALLTHRU
+            default:
                 _VL_VPI_ERROR(__FILE__, __LINE__, "%s: Unsupported format (%s) for %s",
 			      VL_FUNC, VerilatedVpiError::strFromVpiVal(value_p->format), vop->fullname());
 		return 0;
@@ -1761,13 +1791,13 @@ void vpi_get_time(vpiHandle object, p_vpi_time time_p) {
 
 PLI_UINT32 vpi_mcd_open(PLI_BYTE8 *filenamep) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     return VL_FOPEN_S(filenamep,"wb");
 }
 
 PLI_UINT32 vpi_mcd_close(PLI_UINT32 mcd) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     VL_FCLOSE_I(mcd); return 0;
 }
 
@@ -1777,7 +1807,7 @@ PLI_BYTE8 *vpi_mcd_name(PLI_UINT32 mcd) {
 
 PLI_INT32 vpi_mcd_printf(PLI_UINT32 mcd, PLI_BYTE8 *formatp, ...) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     va_list ap;
     va_start(ap,formatp);
     int chars = vpi_mcd_vprintf(mcd, formatp, ap);
@@ -1787,7 +1817,7 @@ PLI_INT32 vpi_mcd_printf(PLI_UINT32 mcd, PLI_BYTE8 *formatp, ...) {
 
 PLI_INT32 vpi_printf(PLI_BYTE8 *formatp, ...) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     va_list ap;
     va_start(ap,formatp);
     int chars = vpi_vprintf(formatp, ap);
@@ -1797,14 +1827,14 @@ PLI_INT32 vpi_printf(PLI_BYTE8 *formatp, ...) {
 
 PLI_INT32 vpi_vprintf(PLI_BYTE8* formatp, va_list ap) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     return VL_VPRINTF(formatp, ap);
 }
 
 PLI_INT32 vpi_mcd_vprintf(PLI_UINT32 mcd, PLI_BYTE8 *format, va_list ap) {
     VerilatedVpiImp::assertOneCheck();
     FILE* fp = VL_CVT_I_FP(mcd);
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     // cppcheck-suppress nullPointer
     if (VL_UNLIKELY(!fp)) return 0;
     int chars = vfprintf(fp, format, ap);
@@ -1813,7 +1843,7 @@ PLI_INT32 vpi_mcd_vprintf(PLI_UINT32 mcd, PLI_BYTE8 *format, va_list ap) {
 
 PLI_INT32 vpi_flush(void) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     Verilated::flushCall();
     return 0;
 }
@@ -1821,7 +1851,7 @@ PLI_INT32 vpi_flush(void) {
 PLI_INT32 vpi_mcd_flush(PLI_UINT32 mcd) {
     VerilatedVpiImp::assertOneCheck();
     FILE* fp = VL_CVT_I_FP(mcd);
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!fp)) return 1;
     fflush(fp);
     return 0;
@@ -1840,21 +1870,21 @@ PLI_INT32 vpi_chk_error(p_vpi_error_info error_info_p) {
     if (error_info_p && _error_info_p) {
       *error_info_p = *_error_info_p;
     }
-    if (!_error_info_p) return 0; // no error occured
+    if (!_error_info_p) return 0;  // no error occured
     return _error_info_p->level;  // return error severity level
 };
 
 PLI_INT32 vpi_free_object(vpiHandle object) {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     return vpi_release_handle(object);  // Deprecated
 }
 
-PLI_INT32 vpi_release_handle (vpiHandle object) {
+PLI_INT32 vpi_release_handle(vpiHandle object) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_release_handle %p\n",object););
     VerilatedVpiImp::assertOneCheck();
     VerilatedVpio* vop = VerilatedVpio::castp(object);
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     if (VL_UNLIKELY(!vop)) return 0;
     vpi_remove_cb(object);  // May not be a callback, but that's ok
     delete vop;
@@ -1863,7 +1893,7 @@ PLI_INT32 vpi_release_handle (vpiHandle object) {
 
 PLI_INT32 vpi_get_vlog_info(p_vpi_vlog_info vlog_info_p) VL_MT_SAFE {
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     vlog_info_p->argc = Verilated::getCommandArgs()->argc;
     vlog_info_p->argv = (PLI_BYTE8**)Verilated::getCommandArgs()->argv;
     vlog_info_p->product = (PLI_BYTE8*)Verilated::productName();
@@ -1889,7 +1919,7 @@ PLI_INT32 vpi_put_userdata(vpiHandle obj, void *userdata) {
 PLI_INT32 vpi_control(PLI_INT32 operation, ...) {
     VL_DEBUG_IF_PLI(VL_DBG_MSGF("- vpi: vpi_control %d\n",operation););
     VerilatedVpiImp::assertOneCheck();
-    _VL_VPI_ERROR_RESET(); // reset vpi error status
+    _VL_VPI_ERROR_RESET();
     switch (operation) {
     case vpiFinish: {
 	VL_FINISH_MT(__FILE__,__LINE__,"*VPI*");
